package com.jwl.driver.server.config;

import com.github.xiaoymin.knife4j.spring.extension.OpenApiExtensionResolver;
import io.swagger.annotations.ApiOperation;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2WebMvc;

/**
 * @Descript Swagger 配置
 * @Version 0.0.1
 **/

@Configuration
@EnableSwagger2WebMvc
public class SwaggerConfig {

    private final OpenApiExtensionResolver openApiExtensionResolver;
    public SwaggerConfig(OpenApiExtensionResolver openApiExtensionResolver){
        this.openApiExtensionResolver = openApiExtensionResolver;
    }

    @Bean
    @Order(value = 1)
    public Docket adminDocket(){
        return new Docket(DocumentationType.SWAGGER_2)
                .pathMapping("/driver-api")
                .enable(true)
                .apiInfo(groupApiInfo())
                .select()
                .apis(RequestHandlerSelectors.withMethodAnnotation(ApiOperation.class))
                .paths(PathSelectors.any())
                .build()
                .extensions(openApiExtensionResolver.buildExtensions("接口中心"));
    }

    private ApiInfo groupApiInfo(){
        return new ApiInfoBuilder()
                .title("管理中心")
                .description("管理中心接口文档")
                .termsOfServiceUrl("http://127.0.0.1")
                .contact(new Contact("jwl",
                        "http://www.jslx.com",
                        "1149034574@qq.com"))
                .version("1.0")
                .build();
    }
}
