package com.jwl.driver.server.exception;


import com.jwl.driver.server.response.BaseResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.security.SignatureException;


/**
 * 全局异常.
 *
 * @author tangsz
 */

@RestControllerAdvice
public class GlobalExceptionHandler {

    private static final Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @ExceptionHandler(value = Exception.class)
    public BaseResponse<Object> handleBadRequest(Exception e) {
        logger.error("Error: handleBadRequest StackTrace : {}", e);

        if (e instanceof MissTokenException) {
            return BaseResponse.fail("4401", null, e.getMessage());
        }
        if (e instanceof IdentityExpiresException) {
            return BaseResponse.fail("4402", null, e.getMessage());
        }
        return BaseResponse.fail(null, e.getMessage());
    }


    @ExceptionHandler(BusinessException.class)
    public BaseResponse<?> businessException(BusinessException e) {
        return BaseResponse.fail(e.getCode(), null, e.getMessage());
    }

    @ExceptionHandler(SignatureException.class)
    public BaseResponse<?> signatureException(SignatureException e) {
        return BaseResponse.fail("E004", "token is invalid");
    }
}
