package com.jwl.driver.server.generator;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.exceptions.MybatisPlusException;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.generator.AutoGenerator;
import com.baomidou.mybatisplus.generator.InjectionConfig;
import com.baomidou.mybatisplus.generator.config.*;
import com.baomidou.mybatisplus.generator.config.po.TableInfo;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.baomidou.mybatisplus.generator.engine.FreemarkerTemplateEngine;

import java.io.File;
import java.lang.reflect.Field;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * <p>
 *  代码生成器工具
 * </p>
 *
 * @author tangsz
 * @since 2019-05-22
 */

public class CodeGeneratorTool {

  private  String host = "";
  private  String database = "";
  private  String url = "";
  private  String userName = "";
  private  String password = "";
  private  String driverName = "com.mysql.cj.jdbc.Driver";
  private  String projectPath = System.getProperty("user.dir");
  private String schema = "";

  private  String mapperXmlFile = "";


  public void generCode(String host, String database, String url, String userName, String password, String projectPath,
                        String module, String superEntityClass) throws ClassNotFoundException {
    this.host = host;
    this.database = database;
    this.url = url;
    this.userName = userName;
    this.password = password;
    this.projectPath = this.projectPath + projectPath;

    // 代码生成器
    AutoGenerator mpg = new AutoGenerator();

    // 全局配置
    mpg.setGlobalConfig(initGlobalConfig());

    // 数据源配置
    mpg.setDataSource(initDataSourceConfig());

    // 包配置
    PackageConfig pc = new PackageConfig();
    pc.setModuleName(module);
    pc.setModuleName("");
    pc.setParent("");
    // 添加 gen，标识为生成的代码
    pc.setController(module + ".controller");
    pc.setEntity(module + ".entity");
    pc.setMapper(module + ".mapper");
    pc.setService(module + ".service");
    pc.setServiceImpl(module + ".service.impl");

    mpg.setPackageInfo(pc);

    // 自定义配置

    mpg.setCfg(initInjectionConfig());
    mpg.setTemplate(new TemplateConfig().setXml(null));

    // 策略配置
    StrategyConfig strategy = new StrategyConfig();
    strategy.setNaming(NamingStrategy.underline_to_camel);
    strategy.setColumnNaming(NamingStrategy.underline_to_camel);
    strategy.setEntityLombokModel(true);
    // 继承基础类
    if (StringUtils.isNotBlank(superEntityClass)) {
      strategy.setSuperEntityClass(superEntityClass);
      // 基础类拥有的字段，不重复生成
      List<String> superEntityColumns = new ArrayList<>();
      Field[] fields = Class.forName(superEntityClass).getDeclaredFields();
      for (Field field : fields) {
        //在类的外面获取此类的私有成员变量的value时需配置 否则会抛异常
        field.setAccessible(true);
        boolean isIdField = field.isAnnotationPresent(TableField.class);
        if (isIdField) {
          superEntityColumns.add(field.getAnnotation(TableField.class).value());
        }
      }

      if (CollUtil.isNotEmpty(superEntityColumns)) {
        strategy.setSuperEntityColumns(superEntityColumns.toArray(new String[superEntityColumns.size()]));
      }
    }
//    strategy.setSuperControllerClass("com.ai.cloudframe.common.base.support.BaseController");
    String mode = scanner("请输入生成模式，1 - 全表 2 - 单表");
    while (!("1".equals(mode) || "2".equals(mode))) {
      scanner("请输入生成模式，1 - 全表 2 - 单表");
    }
    if ("1".equals(mode)) {
      final String schema = scanner("请输入数据库名");
      this.schema = schema;
      final String[] tables = listAllTable(schema);
      strategy.setInclude(tables);
    } else {
      strategy.setInclude(scanner("表名"));
    }
    strategy.setTablePrefix(scanner("表前缀"));

    mpg.setStrategy(strategy);
    mpg.setTemplateEngine(new FreemarkerTemplateEngine());

    mpg.setTemplate(initTemplateConfig());
    mpg.execute();
  }


  /**
   * 全局配置
   * @return
   */
  private GlobalConfig initGlobalConfig() {
    GlobalConfig gc = new GlobalConfig();
    gc.setOutputDir(projectPath + "/src/main/java");
    gc.setAuthor("Automated procedures");
    gc.setOpen(false);
    return gc;
  }


  /**
   * 配置数据源
   * @return
   */
  private DataSourceConfig initDataSourceConfig() {
    return new DataSourceConfig()
            .setUrl(url)
            .setDriverName(driverName)
            .setUsername(userName)
            .setPassword(password);
  }

  /**
   * 自定义配置
   * @return
   */
  private InjectionConfig initInjectionConfig() {
    InjectionConfig cfg = new InjectionConfig() {
      @Override
      public void initMap() {
        // to do nothing
      }
    };
    List<FileOutConfig> focList = new ArrayList<>();
    focList.add(new FileOutConfig("/templates/mapper.xml.ftl") {
      @Override
      public String outputFile(TableInfo tableInfo) {
        // 自定义输入文件名称
        mapperXmlFile = projectPath + "/src/main/resources/mapper/" + "/" + tableInfo.getEntityName() + "Mapper" + StringPool.DOT_XML;
        return projectPath + "/src/main/resources/mapper/" + "/" + tableInfo.getEntityName() + "Mapper" + StringPool.DOT_XML;
      }
    });
    cfg.setFileOutConfigList(focList);
    return cfg;
  }

  /**
   * 覆盖Entity以及xml
   * @return
   */
  private TemplateConfig initTemplateConfig() {
    TemplateConfig tc = new TemplateConfig();
    tc.setXml(null);
    //如果当前Entity已经存在,那么仅仅覆盖Entity
    File file = new File(mapperXmlFile);
    if (file.exists()) {
      tc.setController(null);
      tc.setMapper(null);
      tc.setService(null);
      tc.setServiceImpl(null);
      tc.setEntityKt(null);
    }
    return tc;
  }

  /**
   * <p>
   * 读取控制台内容
   * </p>
   */
  public  String scanner(String tip) {
    Scanner scanner = new Scanner(System.in);
    StringBuilder help = new StringBuilder();
    help.append("请输入" + tip + "：");
    System.out.println(help.toString());
    if (scanner.hasNext()) {
      String ipt = scanner.next();
      if (StringUtils.isNotEmpty(ipt)) {
        return ipt;
      }
    }
    throw new MybatisPlusException("请输入正确的" + tip + "！");
  }

  private String[] listAllTable(String db) {
    try {
      Class.forName("com.mysql.cj.jdbc.Driver");
    } catch (ClassNotFoundException e) {
      throw new RuntimeException(e);
    }
    List<String> tableNames = new ArrayList<>(20);
    try (Connection connection = DriverManager.getConnection(url, userName, password)) {
      final DatabaseMetaData metaData = connection.getMetaData();
      final ResultSet tables =
          metaData.getTables(db,null, null, new String[] { "TABLE" });
      while (tables.next()) {
        System.out.printf("%s - %s - %s%n", tables.getString(1), tables.getString(2), tables.getString(3));
        if (!schema.equals(tables.getString(1))) {
          System.out.printf("skip %s.%n", tables.getString(3));
        }
        tableNames.add(tables.getString(3));
      }
    } catch (SQLException throwables) {
      throw new RuntimeException(throwables);
    }
    return tableNames.toArray(new String[0]);
  }


}
