package com.jwl.driver.server.interceptor;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.StrUtil;
import com.jwl.driver.server.config.AuthConfig;
import com.jwl.driver.server.constant.ErrorCode;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.redis.RedisCache;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Objects;

/**
 * @author 曹林
 * @description
 * @create 2023/8/11 18:05
 */

@Component
@Slf4j
public class AuthInterceptor implements HandlerInterceptor {

    @Autowired
    private RedisCache redisCache;

    @Resource
    private AuthConfig authConfig;


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String requestURI = request.getRequestURI();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("text/html;charset=utf-8");
        String requestTokenHeader = request.getHeader("Authorization");

        Boolean preAuth = authConfig.getPreAuth();
        ArrayList<String> noAuthEndPoints = ListUtil.toList(authConfig.getNoAuthEndPoints());
        ArrayList<String> needAuthEndPoints = ListUtil.toList(authConfig.getNeedAuthEndPoints());

        if (StrUtil.isBlank(requestTokenHeader)) {
            log.warn("Authorization token 为空");
        }

        // preAuth = false，则所有请求（除需要权限校验url集合needAuthEndPoints外）都不需要身份认证
        if (BooleanUtil.isFalse(preAuth)) {
            Boolean contains = needAuthEndPoints.contains(requestURI);
            // 必须登录的接口 && token为空
            if (contains && StrUtil.isBlank(requestTokenHeader)) {
                throw new BusinessException(ErrorCode.AUTH_ERROR, "尚未授权登录");
            }
        }else {
            // preAuth = true;
            // 配置文件无需登录的接口
            Boolean contains = noAuthEndPoints.contains(requestURI);
            // 具备排除标志的接口
            Boolean excludeFlag = Boolean.FALSE;
            for (String exclude : needAuthEndPoints) {
                if (requestURI.contains(exclude)) {
                    excludeFlag = Boolean.TRUE;
                    break;
                }
            }
            if (!contains && !excludeFlag && StrUtil.isBlank(requestTokenHeader)) {
                throw new BusinessException(ErrorCode.AUTH_ERROR, "尚未授权登录");
            }
        }

        if (StrUtil.isNotBlank(requestTokenHeader)){
            Object loginStatus = redisCache.getCacheObject(requestTokenHeader);
            if( Objects.isNull(loginStatus)){
                throw new BusinessException(ErrorCode.AUTH_ERROR, "登陆已过期");
            }
        }

        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }
}


