package com.jwl.driver.server.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.dto.LoginUserDto;
import com.jwl.driver.server.entity.TdSysUser;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdSysUserMapper;
import com.jwl.driver.server.redis.RedisCache;
import com.jwl.driver.server.service.ITdSysUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.util.TokenThreadUtil;
import com.jwl.driver.server.vo.LoginUserVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.UUID;

/**
 * <p>
 * 用户表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdSysUserServiceImpl extends ServiceImpl<TdSysUserMapper, TdSysUser> implements ITdSysUserService {

    @Autowired
    private RedisCache redisCache;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public LoginUserVo login(LoginUserDto loginUserDto) {
        //todo 这里还需要校验验证码

        LambdaQueryWrapper<TdSysUser> cond = new LambdaQueryWrapper<TdSysUser>()
                .eq(TdSysUser::getPhone,"18255439337");
        //用户不存在则直接注册登陆
        TdSysUser tdSysUser = this.baseMapper.selectOne(cond);
        if (tdSysUser == null){
            tdSysUser = new TdSysUser()
                    .setUserName("车友")
                    .setAvatar("")
                    .setPhone(loginUserDto.getPhone())
                    .setCreateTime(LocalDateTime.now());
            int insert = this.getBaseMapper().insert(tdSysUser);
            if (insert != 1){
                throw new BusinessException("用户注册异常");
            }
        }

        String token = UUID.randomUUID().toString();
        redisCache.setCacheObject(token,"123456",365*24*60*60);
        LoginUserVo loginUserVo = new LoginUserVo();
        BeanUtils.copyProperties(tdSysUser,loginUserVo);
        loginUserVo.setToken(token);

        return loginUserVo;
    }

    @Override
    public Boolean loginOut() {
        String token = TokenThreadUtil.getToken();
        if (StrUtil.isNotBlank(token)){
            redisCache.deleteObject(token);
        }
        return Boolean.TRUE;
    }
}
