package com.jwl.driver.server.util;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.jwl.driver.server.constant.ErrorCode;
import com.jwl.driver.server.dto.SecurityUser;
import com.jwl.driver.server.entity.TdSysUser;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.redis.RedisCache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 获取当前登录人信息
 */
public class SecurityUtil {
    private static final Logger log = LoggerFactory.getLogger(SecurityUtil.class);

    private static final RedisCache redisCache = SpringAsyncUtil.<RedisCache>getBean(RedisCache.class);


    /**
     * 获取当前登录人信息
     * @return
     */
    public static SecurityUser getLoginUser() {
        String token = TokenThreadUtil.getToken();
        if (StrUtil.isBlank(token))
            throw new BusinessException(ErrorCode.AUTH_ERROR, "尚未登录");
        SecurityUser securityUser = null;
        TdSysUser tdSysUser = redisCache.getCacheObject(token);

        if (Objects.isNull(securityUser)){
            throw new BusinessException(ErrorCode.AUTH_ERROR, "登录信息已失效");
        }
        BeanUtils.copyProperties(tdSysUser,securityUser);
        securityUser.setToken(token);

        return securityUser;
    }

    /**
     * 获取当前登录人用户编号
     * @return
     */
    public static Long getUserId() {
        Long userId = null;
        SecurityUser loginUser = SecurityUtil.getLoginUser();
        if (null != loginUser) {
            userId = loginUser.getUserId();
        }

        return userId;
    }


}
